"""Echo command line arguments


"""

__author__ = "Jason Wong"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2011/11/18 10:06:20 $"
__copyright__ = "Copyright (c) 2011 Jason WH Wong"
__license__ = "UNSW Faculty of Medicine"

import sys
import os
import getopt
import tokenize
import StringIO
import gzip
import subprocess
import random

class run_MMSAT:

    def __init__(self):
        self.outFile = None
        self.inputFile = None
        self.data = []
        self.files = []
        self.m_err = 0.25
        self.t_err = 0.5
        self.cutoff = 1000
        self.d = 0
        self.w = 0
        self.n = 1
        self.s = 0
        self.outs= []
        self.runfiles = []

    def readInputFile(self):
         print "Reading input Files..."
         iszip = 0
         newfile = self.inputFile
         newpath = os.path.dirname(newfile)+"//"+os.path.basename(newfile).split(".")[0]
         print newpath
         try:
           if newfile.lower().find('.zip') != -1 or newfile.lower().find('.gz') != -1 or newfile.lower().find('.rar') != -1 or newfile.lower().find('.7z') != -1:
              iszip = 1
              #newpath = os.path.dirname(newfile)+"//"+os.path.basename(newfile).split(".")[0]
              print newpath
              os.system("mkdir "+newpath)
              os.system("unzip "+newfile+" -d "+newpath)
              #os.system("7za e -o"+newpath+" "+newfile)                           ## EDIT
              #os.system("del "+newfile)                                                           ## EDIT
              ls = os.listdir(newpath)
              for i in ls:
                  if i.lower().find('.mzxml') != -1:
                     self.files.append(newpath+"//"+i)
           elif newfile.lower().find('.mzxml') != -1:
              self.files.append(newfile)
           else:  #is directory
              tmpfiles = os.listdir(newfile)
              for i in tmpfiles:
                  self.files.append(self.inputFile+"//"+i)

         except IOError:
               print "Incorrect extensions (may be zip not installed), continue trying..."
               files = os.listdir(self.inputFile)
               for i in files:
                   self.files.append(self.inputFile+"//"+i)
         #print self.files
         print "Number of files: "+str(len(self.files))

         for i in self.files:
           cfile = i
           path = os.path.dirname(newfile)
           outfile = cfile[:cfile.rfind(".")]
           outfile +="_sum.out"
           self.outs.append(outfile)
           executable = "./MMSAT -c -i "+cfile+ " -s "+outfile+" -M "+str(self.m_err)+" -T "+str(self.t_err)+" -C "+str(self.cutoff)
           if self.d ==1:
              executable += " -d "
           if self.w == 1:
              executable += " -l "
           if self.s == 1:
              executable += " -S "
           if self.n == 1:
              executable += " -n "
           self.runfiles.append(executable)
         path_in = path+"//"+os.path.basename(newfile)+".outputs.txt"
         self.make_outputs(path_in,self.outs)
         rand = str(int(random.random()*100000000))
         final_out = path+"//"+os.path.basename(newfile)+"_"+rand+"_compiled.txt"
         final_out_diff = path+"//"+os.path.basename(newfile)+"_"+rand+"_diff.txt"
         final_zip = path+"//"+os.path.basename(newfile)+"_"+rand+"_MMSAT.zip"
         runsum = "python merge_LipidMS.py -i "+path_in+" -o "+final_out+" -d "+final_out_diff+" -m "+str(self.m_err)+" -r "+str(self.t_err)
         self.runfiles.append(runsum)
         path_bat = path+"//"+os.path.basename(newfile)+".bat"
         self.make_bat(path_bat,self.runfiles)
         run = subprocess.Popen("sh "+"".join(path_bat),shell=True).wait()
         run = subprocess.Popen("rm "+"".join(path_bat),shell=True).wait()
         os.system("rm "+path_in)
         if iszip == 1:
            os.system("zip -D "+final_zip+" "+newpath+"//*.out "+final_out+" "+final_out_diff)
            os.system("rm -rf "+newpath+"//")
         elif self.inputFile.lower().find('.mzxml') != -1:
            outfile = self.inputFile[:self.inputFile.rfind(".")]
            outfile +="_sum.out"
            os.system("zip -D "+final_zip+" "+outfile+" "+final_out+" "+final_out_diff)
            os.system("rm -f "+outfile)
         else:
            os.system("zip -D "+final_zip+" "+self.inputFile+"//*.out "+final_out+" "+final_out_diff)
            os.system("rm "+self.inputFile+"//*.out")
         print "\nDone! \nCompiled results in: "+final_zip

    def make_outputs(self,path,outs):
        param = open(path,'w')
        for i in outs:
            param.write(i+"\n")
        param.close()
        
    def make_bat(self,path,files):
        bat = open(path,'w')
        for i in files:
            bat.write(i)
            bat.write("\n")
        bat.close()
    
    def ParseCommandLine(self, Arguments):
        try:
            (Options, Args) = getopt.getopt(Arguments, "hdwsni:o:m:r:c:")
        except:
            print "Unknown option entered"
            print UsageInfo
            sys.exit(1)
        OptionsSeen = {}
        for (Option, Value) in Options:
            OptionsSeen[Option] = 1
            if Option == "-i":
                self.inputFile = Value
            elif Option == "-o":
                self.outFile = Value
            elif Option == "-m":
                self.m_err = float(Value)
            elif Option == "-r":
                self.t_err = float(Value)
            elif Option == "-c":
                self.cutoff = int(Value)
            elif Option == "-d":
                self.d = 1
            elif Option == "-w":
                self.w = 1
            elif Option == "-n":
                self.n = 0
            elif Option == "-s":
                self.s = 1
            else:
                print "** Unknown option:", Option, Value
        if not self.inputFile:
            print "\n* Error: -i missing"
            print UsageInfo
            sys.exit(1)

UsageInfo = """
run_MMSAT.py - executes MMSAT

Parameters:
 -i [FILENAME] Input file or directory (where all files to be analysed are stored)
 -o [FILENAME] Output file
 -m [m_err] mass window (Da)
 -r [t_err] rt window (mins)
 -c [cutoff] cutoff
 -d deisotope
 -w waterloss
 -s SRM/MRM mode
 -n turn off denoising

Example:
  run_MMSAT.py -i in.mzXML.gz -o out_summary.txt -m 0.5 -r 0.5 -c 1000 -s
"""

def Main(ex = None):
    global MAX_RESULTS_FILES_TO_PARSE
    if not ex:
        ex = run_MMSAT()
        ex.ParseCommandLine(sys.argv[1:])
        ex.readInputFile()

if __name__ == "__main__":
    #try:
    #    import psyco
    #    psyco.full()
    #except:
    #    print "psyco not found - running without optimization"
    #TestMain()
    Main()