/***************************************************************************************************
File:		fileHandle.cpp
Created:	2011/11/18
Author:		Jason W H Wong <jason.wong@unsw.edu.au>
	
License:	The stand-alone MMSAT application and its source code is free to academic,
		government and non-profit users for non-commercial use. Use of the MMSAT
		application and source code in part or in whole for commercial use by non-profit 
		and for-profit entities requires a license agreement. Parties interested in 
		commercial should contact jason.wong@unsw.edu.au for permission to use
		the extractIC application or its source code.

Disclaimer:	MMSAT and all associated documents ARE PROVIDED "AS IS" WITHOUT ANY WARRANTY
		OF ANY KIND, EITHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, 
		ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
		FREEDOM FROM INFRINGEMENT, OR THAT extractIC and all associated documents WILL
		BE ERROR FREE. The authur MAKES NO REPRESENTATIONS THAT THE USE OF MMSAT or
		any associated documents WILL NOT INFRINGE ANY PATENT OR PROPRIETARY RIGHTS OF
		THIRD PARTIES. In no event will the authors be liable for any damages, including
		but not limited to direct, indirect, special or consequential damages, arising out 
		of, resulting from, or in any way connected with the use of extractIC or any associated
		documents.
***************************************************************************************************/
#define D2M_VERSION 0.1

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <sys/timeb.h>
#include <ctime>
#include <algorithm>
#include <set>
#include "loadmspectrum.h"
#include <netinet/in.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <dirent.h>
#include <iomanip>

#include "StdAfx.h"
#include "fileHandle.h"
#include "base64.h"


fileHandle::fileHandle(){
  mid = 0;
}


fileHandle::~fileHandle(){

}

int fileHandle::importFile(char *_filename, int isMZXML)
{
  string strValue = _filename;
  filename = _filename;
  input_file = _filename;
  string::size_type pos = filename.find_last_of("/");
  if (pos != string::npos){
     filename.erase(0,pos+1);
  }
	FILE *pStream;
	pStream = fopen(_filename,"r");
	char *pValue = new char[1024];
	size_t tRead = 256;
	if(pStream)	{
		tRead = fread(pValue,1,256,pStream);
	}
	if(pStream)	{
		if(pValue[0] == 1 && pValue[1] == -95 || (pValue[3] == 'F' && pValue[5] == 'i' && pValue[7] == 'n') )	{
			cout << "\nFailed to read spectrum file: " << strValue.c_str() << "\n";
			cout << "Most likely cause: using a Finnigan raw spectrum.\nUse ASCII formatted XML files ONLY!\n\n";
			cout.flush();
			delete pValue;
			return false;
		}
		else	{
			size_t d = 0;
			while(d < tRead)	{
				if(pValue[d] == '\0')	{
					cout << "\nFailed to read spectrum file: " << strValue.c_str() << "\n";
					cout << "Most likely cause: using a binary spectrum file.\nUse ASCII formatted XML files ONLY!\n\n";
					cout.flush();
					delete pValue;
					return false;
				}
				d++;
			}
		}
		fclose(pStream);
	}
	delete pValue;


// Start section of code from XTandem
  if(isMZXML == 1 || (strstr(_filename,"mzXML") || strstr(_filename,"MZXML") || strstr(_filename,"XML") || strstr(_filename,"xml") && strstr(_filename,"mzdata.xml")==0))	{
    loadmzxml ldMzxml(all, head);
    if(ldMzxml.open(strValue))	{
      ldMzxml.get();
	  m_tSpectraTotal = all.size();
    }

  }

  else	{
   loadmzdata ldMzdata(all,head);
    if(ldMzdata.open(strValue))  {
      ldMzdata.get();
	  m_tSpectraTotal = all.size();
    }
  }
  // detect number of channels
  cout << "Scans: " << all.size() << endl;
    for (unsigned int i = 0; i < all.size(); ++i){
         vector <string> vals;
         Tokenize(all.at(i).m_bstrFilter,vals," ");
         bool found = false;
         //cout << vals.size() << endl;
         double mass = 0;
         double collision = 0;
         double lowMass =0 ;
         double highMass =0 ;
         if (!SRM){
            if (vals.size() > 6){
             string::size_type at = vals.at(5).find("@");
             if (at != string::npos){
               mass = atof(vals.at(5).substr(0,at-1).c_str());
               collision = atof(vals.at(5).substr(at+1).c_str());
             }
             else{
               mass = atof(vals.at(5).substr(0,at-1).c_str());
             }
             at = vals.at(6).find("-");
             if (at != string::npos){
               at = vals.at(6).find("-");
               lowMass = atof(vals.at(6).substr(1,at-1).c_str());
               highMass = atof(vals.at(6).substr(at+1,vals.at(6).size()-1).c_str());
             }
            }
         }
         else{
           if (vals.size() > 6){
             int mpos = -1;
             for (unsigned int j = 0; j < vals.size(); ++j){
                 if (vals.at(j).compare("ms2") == 0){
                    mpos = j+1;
                 }if (vals.at(j).compare("SRM") == 0){
                    mpos = j+2;
                 }
             }
               int val = vals.size();
               string::size_type at = vals.at(mpos).find("@");
               if (at != string::npos){
                 mass = atof(vals.at(mpos).substr(0,at-1).c_str());
                 string::size_type at2 = vals.at(mpos).find("cid");
                 if (at2 != string::npos){
                    collision = atof(vals.at(mpos).substr(at2+3).c_str());
                    cout << collision << endl;
                 } else{
                   collision = atof(vals.at(5).substr(at+1).c_str());
                 }
               }
               else{
                 mass = atof(vals.at(mpos).c_str());
                 collision = 0;
               }
                //atof(vals.at(5).substr(at+1).c_str());
               at = vals.at(mpos+1).find("-");
               if (at != string::npos){
                 lowMass = atof(vals.at(mpos+1).substr(1,at-1).c_str());
                 highMass = atof(vals.at(mpos+1).substr(at+1,vals.at(mpos+1).size()-1).c_str());
               }
           }  
           if (all.at(i).m_precursorMz != 0){
              mass = all.at(i).m_precursorMz;
           }
           if (lowMass == 0)
              lowMass = all.at(i).m_lowMass;
           if (highMass == 0)
              highMass = all.at(i).m_highMass;
         }
           //cout << mass << "\t" << collision<< "\t" << lowMass << "\t" << highMass << endl;
           for (unsigned int j =0; j < channels.size() && !found; ++j){
               if (channels.at(j).mass == mass && channels.at(j).collision == collision && channels.at(j).lowMass == lowMass && channels.at(j).highMass == highMass){
                  found = true;
                  channels.at(j).data.push_back(&all.at(i));
               }
           }
           if (!found){
              channel c;
              c.mass = mass;
              c.collision = collision;
              c.lowMass = lowMass;
              c.highMass = highMass;
              c.data.push_back(&all.at(i));
              channels.push_back(c);
           }
    }
         cout << "Transitions: " << channels.size() << endl;
         for (unsigned int j =0; j < channels.size(); ++j){
             cout << channels.at(j).mass << "@" << channels.at(j).collision << " [" << channels.at(j).lowMass << "-" << channels.at(j).highMass << "] Scans:" << channels.at(j).data.size() << endl;
         }
// end section of code from XTandem.

  return 1;
}




int fileHandle::importFolder(char *_path, int isMZXML)
{
  struct stat fileStat;
  char szPathName[1024];
  DIR *dirStream;
  struct dirent *dirEntry;
  if (!stat(_path, &fileStat))
  {
         if (S_ISDIR(fileStat.st_mode))
         {
            char *fname;
            if (*(_path + strlen(_path)) != '/')
            {
               // Append separator
               sprintf(szPathName, "%s%c", _path, '/');
            }
            else
            {
               strcpy(szPathName, _path);
            }
            if (!strcmp(szPathName, "./") || !strcmp(szPathName,".\\"))
               szPathName[0]='\0';

            dirStream = (DIR *) opendir(_path);
            while ((dirEntry = readdir((DIR *) dirStream)))
            {
                fname=dirEntry->d_name;
                string fullpath = szPathName;
                fullpath+=fname;
              //  cout <<  fname << endl;
                if (strstr(fullpath.c_str(), ".mzXML") || strstr(fullpath.c_str(), "xml"))
               {
                     importFile((char*)fullpath.c_str(),isMZXML);
               }
            }
         }
         else{
            importFile(_path,isMZXML);
         }
  }
  return 1;
}

int fileHandle::addPeaks(string strPeaks){
    cout << "Peaks: " << strPeaks << endl << flush;
    vector <string> tmpP;
    Tokenize(strPeaks,tmpP,",\n");
    for (unsigned int i = 0; i < tmpP.size(); ++i){
        cout << tmpP.at(i) << endl;
        datapeak p;
        p.dMass = atof(tmpP.at(i).c_str());
        peaks.push_back(p);
    }
    return 1;
}

int fileHandle::importPeaks(char* _filename){
    string strPeaks ="";
    ifstream in((const char*)_filename);
    char buffer[65536]="";
    if (in.is_open()){
		if (! in.eof()){
			in.getline (buffer,65536);
			if (!iswalnum(buffer[0]) && !iswpunct(buffer[0]) && !iswspace(buffer[0]) && !iswdigit(buffer[0])){
				cout << "BAD FILE"<<endl;
				return -1;
			}
		}
                while (!in.eof()){
                       string tmp = buffer;
                       strPeaks += tmp+",";
                       in.getline (buffer,65536);
                }
                string tmp = buffer;
                strPeaks += tmp+",";
    }
     addPeaks(strPeaks);

    return 1;
}

int fileHandle::outputXIC(char* _filename){
  ofstream out;
  out.open(_filename,ios::in|ios::trunc);
  for (unsigned int i = 0; i < channels.size(); ++i){
      out << setiosflags(ios::fixed) << setprecision(2) << "Channel:\t" << channels.at(i).mass << "@" << channels.at(i).collision << " [" << channels.at(i).lowMass << "-" << channels.at(i).highMass << "]\tScans:\t" << channels.at(i).data.size() << endl;
      if  (!channels.at(i).XIC.empty()){
        out << "Mass\tIntensity\tRT(mins)\tScan\tArea\tTIC\t";
        for (unsigned int j = 0; j < channels.at(i).XIC.front().RT.size(); ++j){
            out << setiosflags(ios::fixed) << setprecision(2)<<channels.at(i).XIC.front().RT.at(j)/60 << "\t";
        }
        out << endl;
        for (unsigned int j = 0; j < channels.at(i).XIC.size(); ++j){
          out << setiosflags(ios::fixed) << setprecision(2) <<channels.at(i).XIC.at(j).baseMass<<"\t" <<channels.at(i).XIC.at(j).baseIntn << "\t"<< channels.at(i).XIC.at(j).baseRT/60 <<"\t" <<channels.at(i).XIC.at(j).baseScan<< "\t" << channels.at(i).XIC.at(j).Area << "\t" << channels.at(i).XIC.at(j).TIC << "\t";
          for (unsigned int k = 0; k < channels.at(i).XIC.at(j).RT.size(); ++k){
              out << setiosflags(ios::fixed) << setprecision(2)<<channels.at(i).XIC.at(j).IC.at(k) << "\t";
          }
          out << endl;
        }
      }
      out << endl;
  }
  out.close();
  return 1;
}

int fileHandle::exportMap(char* _filename){
  ofstream out;
  out.open(_filename,ios::in|ios::trunc);
  vector <double> bins;
  out <<"\t";
  for (float i = 400; i < 900; i+=1){
    bins.push_back(i);
    out << i << "\t";
  }
  bins.push_back(900.0);
  out << endl;
  for (unsigned int i = 0; i < all.size(); ++i){
    out << setiosflags(ios::fixed) << setprecision(2) <<all.at(i).m_RT<<"\t";
    for (unsigned int j = 0; j < bins.size()-1; ++j){
        bool found = false;
        double total = 0;
        for (unsigned int k =0; k < all.at(i).peaks.size(); ++k){
          if (all.at(i).peaks.at(k).dMass > bins.at(j) and all.at(i).peaks.at(k).dMass <= bins.at(j+1)){
             total += all.at(i).peaks.at(k).dIntensity;
             found = true;
          }
        }
        if (found)
           out << setiosflags(ios::fixed) << setprecision(2) <<total<<"\t";
        else if (!found)
           out << "0.0\t";
    }
    out << endl;
  }
  out.close();
}

int fileHandle::exportSum(char* _filename, float t_err, float m_err, float cutoff){
  vector <tuple> tmp_all;
  for (unsigned int i = 0; i < channels.size(); ++i){
      for (unsigned int j = 0; j < channels.at(i).XIC.size(); ++j){
          tuple t;
          if (SRM)
             t.mass = channels.at(i).mass;
          else
             t.mass = channels.at(i).XIC.at(j).baseMass;
          t.RT = channels.at(i).XIC.at(j).baseRT/60;
          t.intn = channels.at(i).XIC.at(j).Area;
          t.ch = &channels.at(i);
          t.peak_max = channels.at(i).XIC.at(j).baseScan;
          t.peak_start = channels.at(i).XIC.at(j).start_scan;
          t.peak_end = channels.at(i).XIC.at(j).end_scan;
          tmp_all.push_back(t);
      }
  }
  for (unsigned int i = 0; i < tmp_all.size(); ++i){
      for (unsigned int j = i+1; j < tmp_all.size(); ++j){
          if (tmp_all.at(i).intn == tmp_all.at(j).intn){
              tmp_all.erase(tmp_all.begin()+j);
              j--;
          }
          else if (tmp_all.at(i).ch->mass == tmp_all.at(j).ch->mass  && tmp_all.at(i).mass > tmp_all.at(j).mass-m_err && tmp_all.at(i).mass < tmp_all.at(j).mass+m_err&& tmp_all.at(i).RT > tmp_all.at(j).RT-t_err && tmp_all.at(i).RT < tmp_all.at(j).RT+t_err){
               if (tmp_all.at(i).intn < tmp_all.at(j).intn){
                 tmp_all.erase(tmp_all.begin()+i);
                 i--;
                 break;
              } else{
                tmp_all.erase(tmp_all.begin()+j);
                j--;
              }
          }
      }
  }

  vector <tuple> unique = tmp_all;

  ofstream out;
  out.open(_filename,ios::in|ios::trunc);;
  out <<"mass\trt\tarea\tpeak max(scan)\tpeak start(scan)\tpeak end(scan)\ttransition\n";
  for (unsigned int i = 0; i < unique.size(); ++i){
    if (unique.at(i).intn > cutoff && unique.at(i).RT > 0){
       out << setiosflags(ios::fixed) <<setprecision(5)<<unique.at(i).mass<<"\t"<<unique.at(i).RT<<"\t"<<unique.at(i).intn<<"\t"<<unique.at(i).peak_max<<"\t"<<unique.at(i).peak_start<<"\t"<<unique.at(i).peak_end<<"\t";
       out << setprecision(1) <<unique.at(i).ch->mass;
       if (unique.at(i).ch->collision > 0)
          out <<"@"<<unique.at(i).ch->collision;
       out <<setprecision(4) <<"["<<unique.at(i).ch->lowMass<<"-"<<unique.at(i).ch->highMass<<"]"<<endl;
    }
  }
  out.close();
}




void fileHandle::Tokenize(const string& str,
                      vector<string>& tokens,
                      const string& delimiters = " ")
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}
