/***************************************************************************************************
File:		fileHandle.h
Created:	2011/6/23
Author:		Jason W H Wong <jason.wong@unsw.edu.au>
	
License:	The stand-alone MMSAT application and its source code is free to academic,
		government and non-profit users for non-commercial use. Use of the MMSAT
		application and source code in part or in whole for commercial use by non-profit 
		and for-profit entities requires a license agreement. Parties interested in 
		commercial should contact jason.wong@unsw.edu.au for permission to use
		the extractIC application or its source code.

Disclaimer:	MMSAT and all associated documents ARE PROVIDED "AS IS" WITHOUT ANY WARRANTY
		OF ANY KIND, EITHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, 
		ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
		FREEDOM FROM INFRINGEMENT, OR THAT extractIC and all associated documents WILL
		BE ERROR FREE. The authur MAKES NO REPRESENTATIONS THAT THE USE OF MMSAT or
		any associated documents WILL NOT INFRINGE ANY PATENT OR PROPRIETARY RIGHTS OF
		THIRD PARTIES. In no event will the authors be liable for any damages, including
		but not limited to direct, indirect, special or consequential damages, arising out 
		of, resulting from, or in any way connected with the use of extractIC or any associated
		documents.
***************************************************************************************************/
#ifndef FILEHANDLE_H
#define FILEHANDLE_H

#include <vector>
#include <string>
//#include </cygwin/usr/include/sys/types.h>

#include "sha1.h"
#include "base64.h"
#include "Header.h"

using namespace std;


struct parameters
{
       int EX_CON;
       int MAX_PAR;
       int EX_SCAN;
       double IC_ERR;
       double CON_P_PER;
       double CON_F_PER;
       double P_MIN_INT;
       double F_MIN_INT;
       double CORR;
       int FTCORR;
       double GEN_ERR;
       double MIN_TIC;
       int MAX_SIC;
       int SIC_WIN;
       parameters ()
       {
         EX_CON = 1;
         MAX_PAR = 5;
         EX_SCAN = 4;
         IC_ERR = 0.3;
         CON_P_PER = 0.25;
         CON_F_PER = 0.15;
         P_MIN_INT = 1;
         F_MIN_INT = 0.1;
         CORR = 0.8;
         FTCORR = 3;
         GEN_ERR = 0.02;
         MIN_TIC = 5;
         MAX_SIC = 100;
         SIC_WIN = 75;
       }
};

struct datapeak
{
	double dMass;
	double dIntensity;
	double rt;
	bool isIso;
	datapeak *mono;
        datapeak()
        {
          dMass = 0;
          dIntensity = 0;
          isIso = false;
        }
};

struct chrom{
  double mass;
  double intn;
  double rt;
  int scan;
  vector <double> RT;
  vector <double> IC;
  vector <int> scanNum;
  double TIC;
  double baseMass;
  double baseRT;
  double baseIntn;
  int baseScan;
  double Area;
  int start_scan;
  int end_scan;
};


struct scan
{
  //long scanNum;
  //int level;
  //long	numPakets;
  //double	RT;
  //double	lowMass;
  //double	highMass ;
  //double	TIC;
  //double	basePeakMass;
  //double	basePeakIntensity;
  //double	preIntn;
  //double	preMZ;

        string m_strDescription;
	long   m_tId;
	long	m_peakCount;
	long	m_scanNum;					//
	string	m_bstrFilter;
	long	m_msLevel;   // level
	string	m_polarity;
	string	m_scanType;
	long	m_numPakets;
	double	m_RT;
	double	m_lowMass;
	double	m_highMass ;
	double	m_TIC;
	double	m_basePeakMass;
	double	m_basePeakIntensity;
	long	m_channel;
	long	m_uniformTime;
	double	m_frequency;
	double	m_CE;	// RT of the survey scan from which the precursor was selected
	double	m_precursorIntensity;
	double	m_precursorMz;				//
	int     m_precursorCharge;			//
	char	*m_pEncoded , *m_pEncodedPrevScan;

	int m_encoding;		// 0 = endian, 1 = precision

	double m_fZ;

  vector <datapeak> peaks;
  vector <datapeak> unassigned;
  double sumPeaksIntn;
  int charge[2];

  int num_peaks;
};

struct channel{
  double mass;
  double collision;
  double lowMass;
  double highMass;
  vector <scan*> data;
  vector <chrom> XIC;
};

struct tuple
{
       double mass;
       double RT;
       double intn;
       channel *ch;
       int peak_max;
       int peak_start;
       int peak_end;
};


class fileHandle
{
 public:

  fileHandle();
  virtual ~fileHandle();

  int importFile(char *_filename, int isMZXML);
  int importFolder(char *_path, int isMZXML);
  int importPeaks(char *_filename);
  int addPeaks(string strPeaks);
  int exportXML(char* _filename);
  int outputXIC(char* _filename);
  int exportMap(char* _filename);
  int exportSum(char* _filename, float t_err, float m_err, float cutoff);

  parameters params;
  
  bool SRM;


  vector <scan> all;
  vector <channel> channels;
  vector <datapeak> peaks;
	double TIC;
	double norm_factor;
	string filename;
	int m_tSpectraTotal;
	int scanGap;
        int buf;
        vector <scan> all_mod;
	//vector <scan*> pri;
	//vector <scan*> sec;

  Header head;


  long m_msLevelPrevScan , m_msLevelFirstScan;
  
  scan *prev_level_s;

  //int64_t* m_index;    // for Cygwin
  vector <off64_t> m_index;  // for GCC linux
  int	m_idxFout;
  
  string outMZXML;
  string input_file;

  
  vector <double> contam_list;
  int mid;
  private:

	void m_prepareHeader();
	void m_writeXmlHeader(ofstream &m_fout);
	void m_prepareScan(ofstream &m_fout, scan &s);
	void m_addScanToIndex(ofstream &m_fout, scan &s);
	void m_writeXmlScan(ofstream &m_fout, scan &s);
	void m_writeScanClosing(ofstream &m_fout);
	void m_writeXmlIndex(ofstream &m_fout);
	void m_writeXmlSha1(ofstream &m_fout);
	
        void Tokenize(const string& str, vector<string>& tokens, const string& delimiters);
};
#endif
