/***************************************************************************************************
File:		filterer.cpp
Created:	2011/11/18
Author:		Jason W H Wong <jason.wong@unsw.edu.au>
	
License:	The stand-alone MMSAT application and its source code is free to academic,
		government and non-profit users for non-commercial use. Use of the MMSAT
		application and source code in part or in whole for commercial use by non-profit 
		and for-profit entities requires a license agreement. Parties interested in 
		commercial should contact jason.wong@unsw.edu.au for permission to use
		the extractIC application or its source code.

Disclaimer:	MMSAT and all associated documents ARE PROVIDED "AS IS" WITHOUT ANY WARRANTY
		OF ANY KIND, EITHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, 
		ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
		FREEDOM FROM INFRINGEMENT, OR THAT extractIC and all associated documents WILL
		BE ERROR FREE. The authur MAKES NO REPRESENTATIONS THAT THE USE OF MMSAT or
		any associated documents WILL NOT INFRINGE ANY PATENT OR PROPRIETARY RIGHTS OF
		THIRD PARTIES. In no event will the authors be liable for any damages, including
		but not limited to direct, indirect, special or consequential damages, arising out 
		of, resulting from, or in any way connected with the use of extractIC or any associated
		documents.
***************************************************************************************************/

#include "filterer.h"
#include <algorithm>
#include <math.h>
#include <iostream>
#include <fstream>
//#include "lm.h"
#include "fileHandle.h"
#include "FastSymmlet8.h"

#include <errno.h>

#define PI	 3.1415926535897932
#define M_2S2LN2 2.354820045030949382
#define DBL_MAX 1.7976931348623158e+308
#define SQ2PI 2.50662827463100050241577
#define ISOTOPE 1.002513
#define sqr(x) ((x)*(x))
#define PEAKWDTH 60.0
//#define ERR      0.75

#if !defined(M_PI)
#define M_PI 3.14159265358979323846
#endif
#if !defined(M_LN2)
#define M_LN2 0.69314718055994530942
#endif

#if !defined(MIN)
#define MIN(X,Y) ((X) <= (Y) ? (X) : (Y))
#endif
#if !defined(MAX)
#define MAX(X,Y) ((X) >= (Y) ? (X) : (Y))
#endif
#if !defined(round)
#define round(x) (x<0?ceil((x)-0.5):floor((x)+0.5))
#endif

filterer::filterer(){
      CUTOFF = 1000;
      m_err = 0.5;
      t_err = 1.0;
      nodenoise = 0;
	//tout.open("dump",ios::in|ios::trunc);
}

filterer::~filterer(){
	//tout.close();
}

// Basic deisotoping algorithm for lipid extracts < 1000Da and only for +1 ions
void filterer::deIsotope(fileHandle &f){
  for (unsigned int x = 0; x < f.all.size(); ++x){
    int sizeB = f.all.at(x).peaks.size();
    vector <datapeak> tmp_spec;
    for (unsigned int i = 0; i < f.all.at(x).peaks.size(); ++i){
      bool isIso = false;
      for (unsigned int j = 0; j < tmp_spec.size(); ++j){
          if (f.all.at(x).peaks.at(i).dIntensity > CUTOFF && f.all.at(x).peaks.at(i).dIntensity > f.all.at(x).m_basePeakIntensity*0.005 && isIsotope(f.all.at(x).peaks.at(i),tmp_spec.at(j))){
            isIso = true;
            if (tmp_spec.at(j).isIso){
                f.all.at(x).peaks.at(i).mono = tmp_spec.at(j).mono;
                tmp_spec.at(j).mono->dIntensity += f.all.at(x).peaks.at(i).dIntensity;
            } else{
                f.all.at(x).peaks.at(i).mono = &tmp_spec.at(j);
                tmp_spec.at(j).dIntensity += f.all.at(x).peaks.at(i).dIntensity;
            }
            f.all.at(x).peaks.at(i).isIso = true;
            //cout << f.all.at(x).m_scanNum << " " << f.all.at(x).peaks.at(i).dMass << " " << f.all.at(x).peaks.at(i).mono->dMass << endl;
          }
      }
      tmp_spec.push_back(f.all.at(x).peaks.at(i));
    }
    f.all.at(x).peaks.clear();
    for (unsigned int i = 0; i < tmp_spec.size(); ++i){
        if (!tmp_spec.at(i).isIso){
           f.all.at(x).peaks.push_back(tmp_spec.at(i));
        }
    }
    tmp_spec.clear();
    if (sizeB-f.all.at(x).peaks.size() > 0){
       cout << "Scan: "<< f.all.at(x).m_scanNum << " Peaks after deisotoping: " << f.all.at(x).peaks.size()<< " (-" << sizeB-f.all.at(x).peaks.size() << ") " << endl;
    }
  }
}

bool filterer::isIsotope(datapeak &m1, datapeak &m2){                             // m2 is the parent
     double offset = m_err;
     if (m1.dMass+offset > m2.dMass+ISOTOPE && m1.dMass-offset < m2.dMass+ISOTOPE){
           if (m1.dIntensity < m2.dIntensity){
              return true;
           }
     }
     if (m1.dMass+offset > m2.dMass+ISOTOPE/2 && m1.dMass-offset < m2.dMass+ISOTOPE/2){
           if (m1.dIntensity < m2.dIntensity){
              return true;
           }
     }
     if (m1.dMass+offset > m2.dMass+ISOTOPE/3 && m1.dMass-offset < m2.dMass+ISOTOPE/3){
           if (m1.dIntensity < m2.dIntensity){
              return true;
           }
     }
     return false;
}

// Basic removal of Water and Oxygen loss for +1 ions
void filterer::removeH2O(fileHandle &f){
     for (unsigned int x = 0; x < f.all.size(); ++x){
         int sizeB = f.all.at(x).peaks.size();
         vector <datapeak> tmp_spec;
         for (int i = f.all.at(x).peaks.size()-1; i >-1; --i){
            bool isLoss = false;
            for (unsigned int j = 0; j < tmp_spec.size(); ++j){
                if (f.all.at(x).peaks.at(i).dIntensity > CUTOFF && f.all.at(x).peaks.at(i).dIntensity > f.all.at(x).m_basePeakIntensity*0.005 && (isOLoss(f.all.at(x).peaks.at(i),tmp_spec.at(j)) || isH2OLoss(f.all.at(x).peaks.at(i),tmp_spec.at(j)))){
                    isLoss = true;
                    tmp_spec.at(j).dIntensity += f.all.at(x).peaks.at(i).dIntensity;
                }
                //cout << f.all.at(x).m_scanNum << " " << f.all.at(x).peaks.at(i).dMass << " " << tmp_spec.at(j).dMass << endl;
            }
            if (!isLoss){
               tmp_spec.push_back(f.all.at(x).peaks.at(i));
            }
         }
         f.all.at(x).peaks.clear();
         for (int i = tmp_spec.size()-1; i >-1; --i){
               f.all.at(x).peaks.push_back(tmp_spec.at(i));
         }
         tmp_spec.clear();
         if (sizeB-f.all.at(x).peaks.size() > 0){
            cout << "Scan: "<< f.all.at(x).m_scanNum << " Peaks after Loss: " << f.all.at(x).peaks.size()<< " (-" << sizeB-f.all.at(x).peaks.size() << ") " << endl;
         }
     }
}

bool filterer::isOLoss(datapeak &m1, datapeak &m2){                                 // m2 is the parent
     double offset = 0.5;
     if (m1.dMass+offset > m2.dMass-16 && m1.dMass-offset < m2.dMass-16){
           if (m1.dIntensity < m2.dIntensity){
              return true;
           }
     }
     return false;
}

bool filterer::isH2OLoss(datapeak &m1, datapeak &m2){                                 // m2 is the parent
     double offset = 0.5;
     if (m1.dMass+offset > m2.dMass-18 && m1.dMass-offset < m2.dMass-18){
           if (m1.dIntensity < m2.dIntensity){
              return true;
           }
     }
     return false;
}


void filterer::generateXIC(fileHandle &f){
  for (unsigned int i = 0; i < f.channels.size(); ++i){
      bool hasPeaks = false;
      for (unsigned int j = 0; j < f.channels.at(i).data.size() &&!hasPeaks; ++j){
         vector <datapeak> curPeaks = f.peaks;
         if (curPeaks.empty()){
            pickPeaks(f.channels.at(i).data.at(j),curPeaks);
         } else{
            hasPeaks = true;
         }
         for (unsigned int k= 0; k < curPeaks.size(); ++k){
             chrom newChrom;
             newChrom.mass = curPeaks.at(k).dMass;
             newChrom.intn = curPeaks.at(k).dIntensity;
             newChrom.rt = f.channels.at(i).data.at(j)->m_RT;
             newChrom.scan = f.channels.at(i).data.at(j)->m_scanNum;
             newChrom.TIC = 0;
             for (unsigned int l = 0; l < f.channels.at(i).data.size(); ++l){
                 newChrom.RT.push_back(f.channels.at(i).data.at(l)->m_RT);
                 newChrom.scanNum.push_back(f.channels.at(i).data.at(l)->m_scanNum);
                 double IC = 0;
                 for (unsigned int m = 0; m <f.channels.at(i).data.at(l)->peaks.size(); ++m){
                   if (curPeaks.at(k).dMass > f.channels.at(i).data.at(l)->peaks.at(m).dMass-m_err &&curPeaks.at(k).dMass < f.channels.at(i).data.at(l)->peaks.at(m).dMass+m_err){
                     IC+= f.channels.at(i).data.at(l)->peaks.at(m).dIntensity;
                   }
                 }
                 newChrom.IC.push_back(IC);
                 newChrom.TIC+=IC;
             }
             //SGsmooth(13,newChrom);
             f.channels.at(i).XIC.push_back(newChrom);
         }
      }
  }
}

void filterer::pickPeaks(scan *s, vector <datapeak> &peaks){
     for (unsigned int i = 0; i < s->peaks.size(); ++i){
         if (s->peaks.at(i).dIntensity > CUTOFF && s->peaks.at(i).dIntensity > s->m_basePeakIntensity*0.025){
                   s->peaks.at(i).rt = s->m_RT;
                   peaks.push_back(s->peaks.at(i));
         }
     }
}

void filterer::cullXIC(fileHandle &f){
  for (unsigned int i = 0; i < f.channels.size(); ++i){
      //cout << "BEFORE cull: " <<f.channels.at(i).mass <<"  "<<f.channels.at(i).XIC.size() <<endl;
      vector <chrom> selectXIC;
      // Culling same or similar XICs
      for (unsigned int j = 0; j < f.channels.at(i).XIC.size(); ++j){
           bool cull = false;
           for (unsigned int k = 0; k < selectXIC.size() && !cull; ++k){
               if (f.channels.at(i).XIC.at(j).baseMass > selectXIC.at(k).baseMass-m_err && f.channels.at(i).XIC.at(j).baseMass < selectXIC.at(k).baseMass+m_err && f.channels.at(i).XIC.at(j).baseRT > selectXIC.at(k).baseRT-(t_err*60) && f.channels.at(i).XIC.at(j).baseRT < selectXIC.at(k).baseRT+(t_err*60)){
                  cull = true;
                  //cout << "CULLED" << endl;
                  if (f.channels.at(i).XIC.at(j).Area > selectXIC.at(k).Area){
                    selectXIC.at(k) = f.channels.at(i).XIC.at(j);
                  }
               }
           }
           if (!cull && f.channels.at(i).XIC.at(j).Area > CUTOFF*5){      // to remove single spurious peaks
              selectXIC.push_back(f.channels.at(i).XIC.at(j));
           }
      }
      f.channels.at(i).XIC = selectXIC;
      //cout << "AFTER cull: " <<f.channels.at(i).XIC.size() << endl;
      /*for (unsigned int j = 0; j < f.channels.at(i).XIC.size(); ++j){
            if (f.channels.at(i).mass == 814.7){
                cout <<  f.channels.at(i).XIC.at(j).baseMass <<"\t" << f.channels.at(i).XIC.at(j).baseRT/60 << "\t" << f.channels.at(i).XIC.at(j).Area <<"\t" << endl;
            }
      }*/
  }
}

void filterer::processXIC(fileHandle &f){
  for (unsigned int i = 0; i < f.channels.size(); ++i){
      for (unsigned int j = 0; j < f.channels.at(i).XIC.size(); ++j){
         f.channels.at(i).XIC.at(j).baseRT = 0;
         f.channels.at(i).XIC.at(j).baseIntn = 0;
         chrom tmp = f.channels.at(i).XIC.at(j);
         //
         //
         double sens = 0.15;
         if (!nodenoise){
            waveletDenoise(sens,10,f.channels.at(i).XIC.at(j));
         }
         SGsmooth(3,f.channels.at(i).XIC.at(j));
         int maxPos = 0;
         //rt is in seconds
         //need to find local maximum. First find k for current rt
         int rtScan = -1;
         bool found = false;
         for (unsigned int k = 0; k < f.channels.at(i).XIC.at(j).RT.size()&&!found; ++k){
              if (f.channels.at(i).XIC.at(j).rt == f.channels.at(i).XIC.at(j).RT.at(k)){
                 rtScan = k;
                 //cout << rtScan << endl;
                 found = true;
              }
         }
         double leftMaxIntn = 0;
         int leftMaxScan = 0;
         double leftMaxRT = 0;
         int leftK = rtScan;
         int end = 0;
         for (int k = rtScan; k >-1 && end < 2; --k){
             if (f.channels.at(i).XIC.at(j).IC.at(k) > leftMaxIntn){
               leftMaxScan = f.channels.at(i).XIC.at(j).scanNum.at(k);
               leftMaxIntn = f.channels.at(i).XIC.at(j).IC.at(k);
               leftMaxRT =  f.channels.at(i).XIC.at(j).RT.at(k);
               leftK = k;
             }
             else if (f.channels.at(i).XIC.at(j).IC.at(k) < leftMaxIntn){
                 end++;
             } else if (leftMaxIntn < CUTOFF){
                end = 3;
             }
         }
         double rightMaxIntn = 0;
         int rightMaxScan = f.channels.at(i).XIC.at(j).scanNum.at(f.channels.at(i).XIC.at(j).RT.size()-1);
         double rightMaxRT = 0;
         int rightK = rtScan;
         end = 0;
         for (int k = rtScan; k < f.channels.at(i).XIC.at(j).RT.size() && end < 2; ++k){
             if (f.channels.at(i).XIC.at(j).IC.at(k) > rightMaxIntn){
               rightMaxScan = f.channels.at(i).XIC.at(j).scanNum.at(k);
               rightMaxIntn = f.channels.at(i).XIC.at(j).IC.at(k);
               rightMaxRT = f.channels.at(i).XIC.at(j).RT.at(k);
               rightK = k;
             }
             else if (f.channels.at(i).XIC.at(j).IC.at(k) < rightMaxIntn) {
                 end++;
             } else if (rightMaxIntn < CUTOFF){
                end = 3;
             }
         }
         if (rtScan == leftK || (rtScan-leftK >rightK-rtScan && rightMaxIntn > 0 && rightK != rtScan)){           // left is further that right
             tmp.baseIntn =rightMaxIntn;
             tmp.baseScan = rightMaxScan;
             tmp.baseRT = rightMaxRT;
             maxPos = rightK;
         }
         else{
             tmp.baseIntn =leftMaxIntn;
             tmp.baseScan = leftMaxScan;
             tmp.baseRT = leftMaxRT;
             maxPos = leftK;
         }
         //cout << f.channels.at(i).XIC.at(j).baseIntn << " " << f.channels.at(i).XIC.at(j).baseRT << endl;
         double maxIntn = 0;
         tmp.baseMass = f.channels.at(i).XIC.at(j).mass;
         // finds the accurate baseMass
         /*for (unsigned int l = 0; l < f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.size(); ++l){
             if (f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.at(l).dIntensity > maxIntn && f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.at(l).dMass > f.channels.at(i).XIC.at(j).mass-0.5 && f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.at(l).dMass < f.channels.at(i).XIC.at(j).mass+0.5){
                f.channels.at(i).XIC.at(j).baseMass = f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.at(l).dMass;
                maxIntn =f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(maxPos)-1).peaks.at(l).dIntensity;
             }
         } */
         /*for (int k = rtScan; k < f.channels.at(i).XIC.at(j).RT.size(); ++k){
             if (f.channels.at(i).XIC.at(j).IC.at(k) > f.channels.at(i).XIC.at(j).baseIntn){
               f.channels.at(i).XIC.at(j).baseIntn = f.channels.at(i).XIC.at(j).IC.at(k);
               f.channels.at(i).XIC.at(j).baseRT = f.channels.at(i).XIC.at(j).RT.at(k);
               f.channels.at(i).XIC.at(j).baseScan = f.channels.at(i).XIC.at(j).scanNum.at(k);
               double maxIntn = 0;
               f.channels.at(i).XIC.at(j).baseMass = 0;
               // finds the accurate baseMass
               for (unsigned int l = 0; l < f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.size(); ++l){
                   if (f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dIntensity > maxIntn && f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass > f.channels.at(i).XIC.at(j).mass-0.5 && f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass < f.channels.at(i).XIC.at(j).mass+0.5){
                      f.channels.at(i).XIC.at(j).baseMass = f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass;
                      maxIntn =f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dIntensity;
                   }
               }
               maxPos = k;
             }
         }*/

         if (tmp.baseIntn != 0 && tmp.baseMass != 0){
           // CALCULATED AREA UNDER curve
           // First find start and end points of curve
           int k = maxPos;
           int start;
           int end;
           int l = 0;
           while (maxPos+l-1 > 0 && maxPos+l+1 < f.channels.at(i).XIC.at(j).IC.size()&& f.channels.at(i).XIC.at(j).IC.at(maxPos+l) > f.channels.at(i).XIC.at(j).IC.at(maxPos+l-1) && tmp.IC.at(maxPos+l) > CUTOFF/500 && f.channels.at(i).XIC.at(j).RT.at(maxPos)-f.channels.at(i).XIC.at(j).RT.at(maxPos+l) <= PEAKWDTH/2.0){
                  l--;
           }
           start = maxPos+l-1;
           if (start < 0)
              start = 0;
           l = 0;
           while (maxPos+l > 0 &&maxPos+l+1 < f.channels.at(i).XIC.at(j).IC.size() && f.channels.at(i).XIC.at(j).IC.at(maxPos+l) > f.channels.at(i).XIC.at(j).IC.at(maxPos+l+1) && tmp.IC.at(maxPos+l) > CUTOFF/500 && f.channels.at(i).XIC.at(j).RT.at(maxPos+l)-f.channels.at(i).XIC.at(j).RT.at(maxPos) <= PEAKWDTH/2.0){
                  l++;
           }
           end = maxPos+l+1;
           if (end > f.channels.at(i).XIC.at(j).IC.size()){
             end = f.channels.at(i).XIC.at(j).IC.size();
           }
           trapizoid(tmp, start, end);
           f.channels.at(i).XIC.at(j) = tmp;
           f.channels.at(i).XIC.at(j).start_scan = f.channels.at(i).XIC.at(j).scanNum.at(start);
           f.channels.at(i).XIC.at(j).end_scan = f.channels.at(i).XIC.at(j).scanNum.at(end-1);
           f.channels.at(i).XIC.at(j).baseScan = f.channels.at(i).XIC.at(j).scanNum.at(maxPos);
           double sumMass = 0;
           double count = 0;
           for (unsigned int k = start; k < end; ++k){
             for (unsigned int l = 0; l < f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.size(); ++l){
               if (f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass > f.channels.at(i).XIC.at(j).mass-m_err && f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass < f.channels.at(i).XIC.at(j).mass+m_err){
                  sumMass += f.all.at(f.channels.at(i).XIC.at(j).scanNum.at(k)-1).peaks.at(l).dMass;
                  count++;
               }
             }
           }
           if (count > 0)
            f.channels.at(i).XIC.at(j).baseMass = sumMass/count;
         } else{
               f.channels.at(i).XIC.at(j).baseIntn = 0;
               f.channels.at(i).XIC.at(j).baseRT = 0;
               f.channels.at(i).XIC.at(j).baseScan = 0;
               f.channels.at(i).XIC.at(j).baseMass = f.channels.at(i).XIC.at(j).mass;
         }
      }
  }
}

void filterer::trapizoid(chrom &c, int start, int end){
  if (start == end){
     c.Area = c.IC.at(start);
     return;
  }
  double area = 0;
  for (int i = start+1; i < end; ++i){
       area += (c.RT.at(i)-c.RT.at(i-1))*(c.IC.at(i)+c.IC.at(i-1));
  }
  c.Area = area*0.5;
}

void filterer::SGsmooth(int pass, chrom &rough)
{
        if (rough.IC.size() < 12)
           return;
  	static const int l_norm = 21;					// constants used for SG smoothing
	static const int m_norm = 323;
	static const int h_norm = 5175;
	int filter[50];
	int h;
	int norm;
	if (pass <= 5){
		const int l_filter[] = {-2,3,6,7,6,3,-2};
		h = 3;
		norm = l_norm;
		for (unsigned int i = 0; i < h*2+1; ++i)
			filter[i] = l_filter[i];
	}
	else if (pass > 5 && pass <= 10){
		const int m_filter[] = {-21,-6,7,18,27,34,39,42,43,42,39,34,27,18,7,-6,-21};
		h = 8;
		norm = m_norm;
		for (unsigned int i = 0; i < h*2+1; ++i)
			filter[i] = m_filter[i];
	}
	else{
		const int h_filter[] = {-253,-138,-33,62,147,222,287,322,387,422,447,462,467,
					         462,447,422,387,322,287,222,147,62,-33,-138,-253};
		h = 12;
		norm = h_norm;
		for (int i = 0; i < h*2+1; ++i)
			filter[i] = h_filter[i];
	}
	vector <double> sInput;
	for (unsigned int i = 0; i < h; ++i){
		sInput.push_back(rough.IC.at(i));
	}
	double sum = 0;
	for (unsigned int i = h; i < rough.IC.size()-h; ++i){
		sum = 0;
		for (int j = -h; j < h+1; ++j){
			sum += filter[j+h]*(rough.IC.at(i+j));
		}
		sInput.push_back(sum/norm);
	}
	for (unsigned int i = 0; i < h; ++i){
		sInput.push_back(rough.IC.at(rough.IC.size()-h));
	}
	for (unsigned int i = 0; i < rough.IC.size(); ++i){
	    rough.IC.at(i) = sInput.at(i);
	    if (rough.IC.at(i) < 0)
	       rough.IC.at(i) = 0;
        }
}

void filterer::waveletDenoise(double &sens, int recurs, chrom &rough)
{
        vector <double> &input = rough.IC;
	int orgsize = input.size();
	for (unsigned int i = 0; i < recurs; ++i){
		if (i != 0)
			input.insert(input.begin(),(int)((double)orgsize/(double)i),input.front());
		int Esize = input.size();
		int k = 0;
		while (Esize > pow((double)2,(double)k)){
			++k;
		}
		Esize = (int)pow((double)2,(double)k);
		int diff = Esize-input.size();
		input.insert(input.end(),diff,0);
		FastSymmlet8::transform(input,Esize);
		FastSymmlet8::threshold(input,diff,k, sens);
		FastSymmlet8::invTransform(input,Esize);
		input.resize(Esize-diff);
		if (i != 0)
			input.erase(input.begin(),input.begin()+(int)((double)orgsize/(double)i));
	}
}
