/***************************************************************************************************
File:		filterer.h
Created:	2011/6/23
Author:		Jason W H Wong <jason.wong@unsw.edu.au>
	
License:	The stand-alone MMSAT application and its source code is free to academic,
		government and non-profit users for non-commercial use. Use of the MMSAT
		application and source code in part or in whole for commercial use by non-profit 
		and for-profit entities requires a license agreement. Parties interested in 
		commercial should contact jason.wong@unsw.edu.au for permission to use
		the extractIC application or its source code.

Disclaimer:	MMSAT and all associated documents ARE PROVIDED "AS IS" WITHOUT ANY WARRANTY
		OF ANY KIND, EITHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, 
		ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
		FREEDOM FROM INFRINGEMENT, OR THAT extractIC and all associated documents WILL
		BE ERROR FREE. The authur MAKES NO REPRESENTATIONS THAT THE USE OF MMSAT or
		any associated documents WILL NOT INFRINGE ANY PATENT OR PROPRIETARY RIGHTS OF
		THIRD PARTIES. In no event will the authors be liable for any damages, including
		but not limited to direct, indirect, special or consequential damages, arising out 
		of, resulting from, or in any way connected with the use of extractIC or any associated
		documents.
***************************************************************************************************/
#ifndef FILTERER_H
#define FILTERER_H

#include <vector>
#include <string>
#include "fileHandle.h"
#include <fstream>

using namespace std;

class GreaterThanIntn{
public:
	bool operator()(const datapeak &a, const datapeak &b)
	{return a.dIntensity> b.dIntensity;}
};

class GreaterThanIntnS{
public:
	bool operator()(const datapeak *a, const datapeak *b)
	{return a->dIntensity> b->dIntensity;}
};

class LessThanMZ{
public:
	bool operator()(const datapeak &a, const datapeak &b)
	{return a.dMass< b.dMass;}
};

class byScan{
public:
	bool operator()(const scan &a, const scan &b)
	{return a.m_scanNum< b.m_scanNum;}
};


class filterer
{

public:

	filterer();
	virtual ~filterer();

	void computeAllXIC(fileHandle &f, int begin, int end);
	void deIsotope(fileHandle &f);
	void removeH2O(fileHandle &f);
	void generateXIC(fileHandle &f);
        void processXIC(fileHandle &f);
        void cullXIC(fileHandle &f);
        //void extractIC(datapeak &dat, int scan, vector <scan*> &curScans, bool odd, int SIC_WIN, double IC_ERR, double MIN_TIC);
	//void createMSMS(fileHandle &f);
	//void findContam(fileHandle &f);
	int CUTOFF;
	double m_err;
	double t_err;
        int nodenoise;

private:

        void SGsmooth(int pass, chrom &rough);
        bool isIsotope(datapeak &m1, datapeak &m2);
        bool isOLoss(datapeak &m1, datapeak &m2);
        bool isH2OLoss(datapeak &m1, datapeak &m2);
        void pickPeaks(scan *s, vector <datapeak> &peaks);
        void trapizoid(chrom &c, int start, int end);
        void waveletDenoise(double &sens, int recurs, chrom &rough);

};
#endif
