/***************************************************************************************************
File:		main.cpp
Created:	2011/6/23
Author:		Jason W H Wong <jason.wong@unsw.edu.au>
	
License:	The stand-alone MMSAT application and its source code is free to academic,
		government and non-profit users for non-commercial use. Use of the MMSAT
		application and source code in part or in whole for commercial use by non-profit 
		and for-profit entities requires a license agreement. Parties interested in 
		commercial should contact jason.wong@unsw.edu.au for permission to use
		the extractIC application or its source code.

Disclaimer:	MMSAT and all associated documents ARE PROVIDED "AS IS" WITHOUT ANY WARRANTY
		OF ANY KIND, EITHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, 
		ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
		FREEDOM FROM INFRINGEMENT, OR THAT extractIC and all associated documents WILL
		BE ERROR FREE. The authur MAKES NO REPRESENTATIONS THAT THE USE OF MMSAT or
		any associated documents WILL NOT INFRINGE ANY PATENT OR PROPRIETARY RIGHTS OF
		THIRD PARTIES. In no event will the authors be liable for any damages, including
		but not limited to direct, indirect, special or consequential damages, arising out 
		of, resulting from, or in any way connected with the use of extractIC or any associated
		documents.
***************************************************************************************************/

#include <stdio.h>
#include <iostream>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <fstream>
#include "filterer.h"
#include "fileHandle.h"

using namespace std;

int usage();

int main(int argc, char *argv[])
{
  int c;

  char *in_filename = NULL;
  char *out_filename = NULL;
  char *peak_filename = NULL;
  char *map_filename = NULL;
  char *sum_filename = NULL;
  int iflag = 0;
  int oflag = 0;
  int cflag = 0;
  int dflag = 0;
  int Pflag = 0;
  int pflag = 0;
  int Cflag = 0;
  int mflag = 0;
  int sflag =0 ;
  int lflag = 0;
  int Mflag = 0;
  int Tflag = 0;
  int nflag = 0;
  bool Sflag = false;

  double m_err = 0.5;
  double t_err = 0.5;
  int isMZXML = 1;
  int cutoff = 10000;
  int first;
  int last;
  string peaks;

  if (argc == 1) usage();

  while ((c = getopt(argc,argv,"hdclSni:o:p:P:C:m:s:M:T:")) != -1) {
    switch(c){
    case 'M': //mass error
      Mflag = 1;
      m_err = atof(optarg);
      break;
    case 'S':
      Sflag = true;
      break;
    case 'T': //time error
      Tflag = 1;
      t_err = atof(optarg);
      break;
    case 'i': //source file
      iflag = 1;
      in_filename = optarg;
      break;
    case 'o': // destination dir
      oflag = 1;
      out_filename = optarg;
      break;
    case 'm': // destination dir
      mflag = 1;
      map_filename = optarg;
      break;
    case 's': // destination dir
      sflag = 1;
      sum_filename = optarg;
      break;
    case 'd': // deisotope
      dflag = 1;
      break;
    case 'c': // cull
      cflag = 1;
      break;
    case 'C':
      Cflag = 1;
      cutoff = atoi(optarg);
      break;
    case 'n': // don't denoise
      nflag = 1;
      break;
    case 'p':
      pflag = 1;
      peaks = optarg;
      break;
    case 'P':
      Pflag = 1;
      peak_filename = optarg;
      break;
    case 'l':
      lflag = 1;
      break;
    case 'h':  //help
      usage();
    default: 
      usage();
    }
  }
  
  if (iflag == 0){
  	cout << "Please enter input file (-i)."<<endl;
  	exit(0);
  }

  cout << "\nMMSAT: Metabolite Mass Spectrometry Analysis Tool 1.1 (18-Nov-11) \nby Jason W. H. Wong, Faculty of Medicine, \nUniversity of New South Wales, 2011\n"<<endl<<endl;

 	fileHandle fHandle;
 	fHandle.SRM = Sflag;
 	filterer filter;
 	filter.CUTOFF = cutoff;
 	filter.m_err = m_err;
 	filter.t_err = t_err;
 	filter.nodenoise = nflag;
 	cout<<"source: "<<in_filename<<endl;
        cout <<"dest: "<<sum_filename;
 	cout << endl;
    int s;
    s = fHandle.importFolder(in_filename,isMZXML);
    if (s == 0){
                cout << "Cannot read mzXML file"<<endl;
                cout.flush();
		return -1;
    }
    if (pflag){
        s = fHandle.addPeaks(peaks);
    }
    if (Pflag){
        s = fHandle.importPeaks(peak_filename);
    }
    if (dflag && !Sflag)
       filter.deIsotope(fHandle);
    if (lflag&& !Sflag)
       filter.removeH2O(fHandle);

    filter.generateXIC(fHandle);
    filter.processXIC(fHandle);
    if (cflag && !pflag &&!Pflag)
       filter.cullXIC(fHandle);
    if (oflag){
       fHandle.outputXIC(out_filename);
      //filter.compile(fHandle.hits,fHandle.prot,fHandle.num_groups);
      //fHandle.exportProt(prot_filename);
    }
    if (mflag){
       fHandle.exportMap(map_filename);
    }
    if (sflag){
       fHandle.exportSum(sum_filename,t_err,m_err,cutoff);
    }
   cout << "Done"<<endl<<endl;
}

int usage() {
	printf("MMSAT: Metabolite Mass Spectrometry Analysis Tool 1.1 (18-Nov-11) \nby Jason W. H. Wong, Faculty of Medicine, \nUniversity of New South Wales, 2011\n");
	printf("\nUsage:MMSAT <-i source_file> [-o trans_file] [-m map_file] [-s summary_file] \n");
        printf("	        [-p peaks] [-P peak_file] [-d] [-l] [-c] [-S]\n");
        printf("	        [-C cutoff] [-T time_error] [-M mass_error] \n");
	printf("	----------------------------------------------------------------\n");
	printf("       	(source_file)      path of source mzXML file\n");
	printf("       	[trans_file]       path of all transition data\n");
        printf("       	[map_file]         path of binned map of data\n");
        printf("       	[summary_file]     path of summary data\n");
        printf("       	[-S ]   specify SRM mode\n");
        printf("       	[-p ]   list of peaks to quantify\n");
        printf("       	[-P ]   list of peaks in peak_file to quantify\n");
        printf("       	[-d ]   deisotope spectra (for precursor ion/full scanning only)\n");
        printf("       	[-l ]   remove water loss (for precursor ion/full scanning only)\n");
        printf("       	[-c ]   cull similar XICs following peak detection\n");
        printf("       	[-C ]   'cutoff' intensity cutoff for peak detection (10,000)\n");
        printf("        [-M ]   'time_error' time error tolerance to consider two peaks to be the same (0.5 min)\n");
        printf("       	[-T ]   'mass_error' mass error tolerance to consider two peaks to be the same (0.5 Da)\n");
	printf("\nUsage examples: \n");
	printf("./MMSAT  -i /home/guest/raw_data/example.mzXML -s /home/guest/output/example.txt -c -l -d \n");

	exit(1);
}
