/************************************************************
 *              SAXMzdataHandler.cpp
 *
 * Premiere version janvier 2005
 * Patrick Lacasse
 * placasse@mat.ulaval.ca
 *
 * 3/11/2005 (Brendan MacLean): Use eXpat SAX parser, and create SAXSpectraHandler
 *
 * See http://psidev.sourceforge.net/ms/#mzdata for
 * mzData schema information.
 *
 * Inspired by DtaSAX2Handler.cpp
 * copyright            : (C) 2002 by Pedrioli Patrick, ISB, Proteomics
 * email                : ppatrick@systemsbiology.org
 * Artistic License granted 3/11/2005
 *
 * Modified by Jason Wong <jason.wong@ucd.ie> 09-08-2007 for mzData2mzXML
 *
 *******************************************************/

#include "StdAfx.h"
#include "saxmzdatahandler.h"

SAXMzdataHandler::SAXMzdataHandler( vector<scan>& _vS, Header &_head)
	: SAXSpectraHandler(_vS, _head)
{
	m_bInmzArrayBinary=false;
	m_bInintenArrayBinary=false;
	m_bInData=false;
	m_bInMsLevel2=true;
	m_isHeader = false;
	m_isFileType = false;
	m_isInstrumentName = false;
	m_isNameOFile = false;
	m_tId = 0;
	m_precursorCharge = 0;
}

SAXMzdataHandler::~SAXMzdataHandler()
{
}

void SAXMzdataHandler::startElement(const XML_Char *el, const XML_Char **attr)
{
	//Header stuff

	if (isElement("description", el)){

	}
	else if (isElement("fileType", el)){
		m_isFileType = true;
	}
	else if (isElement("instrumentName", el)){
		m_isInstrumentName = true;
	}
	else if (isElement("nameOfFile", el)){
		m_isNameOFile = true;
	}
	else if (isElement("spectrumList", el)){
		scanCount = atoi(getAttrValue("count",attr));
		m_isHeader = true;
	}
	// Let's see what	element	just started
	// We	use:
	//   spectrum	for	the	id number
	//   acqInstrument for the msLevel
	//   cvParam for the polarity	and	the	mz of the parent ion
	//   data	for	the	values of the spectrum

	else if (isElement("spectrum", el))
	{
		m_scanNum =	atoi(getAttrValue("id", attr));
		m_tId++;
	//	while(m_sId.find(m_tId) != m_sId.end())	{
	//		m_tId++;
	//	}
	//	m_sId.insert(m_tId);

	}
	//mzdata v. 1.04 uses acqInstrument and mzdata v. 1.05 uses spectrumInstrument
	else if (isElement("spectrumInstrument", el) || isElement("acqInstrument", el))
	{
		//first	attribute is msLevel
		m_msLevel = atoi(getAttrValue("msLevel", attr));
		reset();
	//	if(	(m_msLevel	= atoi(getAttrValue("msLevel", attr))) == 2 )
	//	{
	//		m_bInMsLevel2 = true;

	//		reset();	// Clean up	for	the	next scan

	//	}
		m_lowMass = atof(getAttrValue("mzRangeStart",attr));
		m_highMass = atof(getAttrValue("mzRangeStop",attr));	
	}
	else if (isElement("cvParam", el))
	{
		const char* name = getAttrValue("name", attr);
		const char* value = getAttrValue("value", attr);
		if (!strcmp(name, "IonizationType")){
			m_ionizationType = value;
		}
		else if (!strcmp(name, "AnalyzerType")){
			m_analyzerType = value;
		}
		else if (!strcmp(name, "DetectorType")){
			m_detectorType = value;
		}
		else if (!strcmp(name, "polarity")){
			if(!strcmp(value, "+"))  
				m_precursorCharge	= 0;
			else{
				m_precursorCharge = atoi(value);
				if(m_precursorCharge ==	0)
					m_precursorCharge = 2;
			}
		}
		else if (!strcmp(name, "mz") )
			m_precursorMz =	atof(value);
		else if (!strcmp(name, "ChargeState")){
			m_precursorCharge = atoi(value);
		}
		else if (!strcmp(name, "MassToChargeRatio") ){
			m_precursorMz =	atof(value);
		}
		else if (!strcmp(name, "Polarity")){
			m_polarity = value;
		}
		else if (!strcmp(name, "TimeInMinutes") || (!strcmp(name, "time.min"))){
			m_RT = atof(value);
		}
		else if (!strcmp(name, "ScanMode")){
			m_scanType = value;
		}
		else if (!strcmp(name, "CollisionEnergy")){
			m_CE = atof(value);
		}
	}
	else if (isElement("mzArrayBinary", el)){
		m_bInmzArrayBinary = true;
		startPeakListBinary(attr);
	}
	else if (isElement("intenArrayBinary", el))	{
		m_bInintenArrayBinary = true;
		startPeakListBinary(attr);
	}
	else if (isElement("data", el))	{
		m_bInData	= true;
		m_peaksCount = atoi(getAttrValue("length", attr));
		if(strlen(getAttrValue("endian", attr)) > 0)	{
			m_bNetworkData = (strcmp("big", getAttrValue("endian", attr)) != 0);
		}
		if(strlen(getAttrValue("precision", attr)) > 0)	{
			m_bLowPrecision = (strcmp("64", getAttrValue("precision", attr)) != 0);
		}
	}
}

void SAXMzdataHandler::startPeakListBinary(const XML_Char **attr)
{
	if(strlen(getAttrValue("endian", attr)) > 0)	{
		m_bNetworkData = (strcmp("little", getAttrValue("endian", attr)) != 0);
	}
	if(strlen(getAttrValue("precision", attr)) > 0)	{
		m_bLowPrecision = (strcmp("64", getAttrValue("precision", attr)) != 0);
	}
}

void SAXMzdataHandler::endElement(const XML_Char *el)
{
	if (isElement("spectrumList", el) && m_isHeader){
		writeHeader();
		m_isHeader = false;
	}
	else if (isElement("nameOfFile", el) && m_isNameOFile){
		m_isNameOFile = false;
	}
	else if (isElement("fileType", el) && m_isFileType){
		m_isFileType = false;
	}
	else if (isElement("instrumentName", el) && m_isInstrumentName){
		m_isInstrumentName = false;
	}
	else if(isElement("mzArrayBinary", el)){
		m_bInmzArrayBinary = false;
	}
	else if(isElement("intenArrayBinary", el)){
		m_bInintenArrayBinary = false;
	}
	else if(isElement("data", el))
	{
		processData();
		m_bInData = false;
	}
	else if(isElement("spectrum", el)) // && m_bInMsLevel2
	{
		pushSpectrum();
		//m_bInMsLevel2 = false;
	}
}

void SAXMzdataHandler::characters(const XML_Char *s, int len)
{
	if (m_isFileType){
		m_fileType.append(s,len);
	}
	else if (m_isNameOFile){
		m_filename.append(s,len);
	}
	else if (m_isInstrumentName){
		m_instrumentName.append(s,len);
	}
	else if((m_bInmzArrayBinary || m_bInintenArrayBinary)  && m_bInData) //&& m_bInMsLevel2
	{
		m_strData.append(s, len);
	}
}

void SAXMzdataHandler::processData()
{
	if((m_bInmzArrayBinary || m_bInintenArrayBinary)  && m_bInData) //&& m_bInMsLevel2
	{
		pushPeaks(m_bInmzArrayBinary, m_bInintenArrayBinary);
	}

	m_strData.clear();
}

