/************************************************************
 * SAXMzxmlHandler.cpp
 *
 * Premiere version janvier 2005
 * Patrick Lacasse
 * placasse@mat.ulaval.ca
 *
 * 3/11/2005 (Brendan MacLean): Use eXpat SAX parser, and create SAXSpectraHandler
 *
 * See http://sashimi.sourceforge.net/software_glossolalia.html for
 * mzXML schema information.
 *
 * Inspired by DtaSAX2Handler.cpp
 * copyright            : (C) 2002 by Pedrioli Patrick, ISB, Proteomics
 * email                : ppatrick@systemsbiology.org
 * Artistic License granted 3/11/2005
 *******************************************************/

#include "stdafx.h"
#include "saxmzxmlhandler.h"
#include <iostream>

SAXMzxmlHandler::SAXMzxmlHandler( vector<scan>& _vS, Header & _head)
	: SAXSpectraHandler(_vS, _head)
{
	m_bInMsLevel2 = false;
	m_bInMsLevel1 = false;
	m_bInPrecursorMz =  false;
	m_bInPeaks = false;
	//added this (true by default)
	m_bNetworkData = false;
	m_isHeader = false;
	m_isFileType = false;
	m_isInstrumentName = false;
	m_isNameOFile = false;
	m_tId = 0;
	m_precursorCharge = 0;
	m_precursorMz = 0;
}

SAXMzxmlHandler::~SAXMzxmlHandler()
{
}

void SAXMzxmlHandler::startElement(const XML_Char *el, const XML_Char **attr)
{
 	//Header stuff

	if (isElement("parentFile", el)){
           m_filename = getAttrValue("fileName", attr);
           m_fileType = getAttrValue("fileType", attr);
	}
	else if (isElement("msModel", el)){
		m_instrumentName = getAttrValue("value", attr);
	}
	else if (isElement("msIonisation", el)){
                m_ionizationType = getAttrValue("value", attr);
	}
	else if (isElement("msMassAnalyzer", el)){
                m_analyzerType = getAttrValue("value", attr);
	}
	else if (isElement("msDetector", el)){
                m_analyzerType = getAttrValue("value", attr);
	}
	if(isElement("scan", el))
	{
		if(atoi(getAttrValue("msLevel", attr)) == 2)
		{
                  	if(m_bInMsLevel1 == true && m_scanType.find("Zoom") == string::npos)
	               {

                            pushSpectrum();
                            m_bInMsLevel1 = false;
                       }
			m_bInMsLevel2 = true;
                        m_msLevel = 2;
			reset();	// Clean up for the next scan

			m_scanNum = atoi(getAttrValue("num", attr));
			m_tId = m_scanNum;
			while(m_sId.find(m_tId) != m_sId.end())	{
				m_tId++;
			}
			m_sId.insert(m_tId);
			m_peaksCount = atoi(getAttrValue("peaksCount", attr));
			m_TIC = atof(getAttrValue("totIonCurrent", attr));
			string tmpRT = getAttrValue("retentionTime", attr);
			tmpRT.resize(tmpRT.size()-1);
			m_RT = atof(tmpRT.substr(2).c_str());
                        m_polarity = getAttrValue("polarity", attr);
                        m_lowMass = atof(getAttrValue("lowMz", attr));
                        m_highMass = atof(getAttrValue("highMz", attr));
                        m_basePeakMass = atof(getAttrValue("basePeakIntensity", attr));
                        m_basePeakIntensity = atof(getAttrValue("totIonCurrent", attr));
                        m_bstrFilter = getAttrValue("filterLine", attr);
		}
		else if(atoi(getAttrValue("msLevel", attr)) == 1 ||atoi(getAttrValue("msLevel", attr)) == 0||atoi(getAttrValue("msLevel", attr)) == -1   /*&& strcmp("Zoom",getAttrValue("scanType", attr))!=0 */)
		{
			m_bInMsLevel1 = true;
			m_msLevel = 1;

			reset();	// Clean up for the next scan

			m_scanNum = atoi(getAttrValue("num", attr));
			m_tId = m_scanNum;
			while(m_sId.find(m_tId) != m_sId.end())	{
				m_tId++;
			}
			m_sId.insert(m_tId);
			m_peaksCount = atoi(getAttrValue("peaksCount", attr));
			m_TIC = atof(getAttrValue("totIonCurrent", attr));

			m_scanType =getAttrValue("scanType", attr);
		//	cout << m_scanNum<< endl;
		//	cout.flush();
			string tmpRT = getAttrValue("retentionTime", attr);
			tmpRT.resize(tmpRT.size()-1);
			m_RT = atof(tmpRT.substr(2).c_str());
                        m_polarity = getAttrValue("polarity", attr);
                        m_lowMass = atof(getAttrValue("lowMz", attr));
                        m_highMass = atof(getAttrValue("highMz", attr));
                        m_basePeakMass = atof(getAttrValue("basePeakIntensity", attr));
                        m_basePeakIntensity = atof(getAttrValue("totIonCurrent", attr));
                        m_bstrFilter = getAttrValue("filterLine", attr);
                        //cout << m_RT << endl;
		}
		else
		{
                        reset();
                }
	}
	else if(isElement("peaks", el))
	{
		m_bInPeaks = true;
		m_bLowPrecision = (strcmp("64", getAttrValue("precision", attr)) != 0);
	}
	else if(isElement("precursorMz", el))
	{
		m_bInPrecursorMz = true;
		m_precursorCharge = atoi(getAttrValue("precursorCharge", attr));
	}
}

void SAXMzxmlHandler::endElement(const XML_Char *el)
{
	if(isElement("peaks", el))
	{
		processData();
		m_bInPeaks = false;
	}
	else if(isElement("precursorMz", el))
	{
		processData();
		m_bInPrecursorMz = false;
	}
	else if (isElement("scan", el) && m_bInMsLevel1 == true && m_scanType.find("Zoom") != string::npos){
                m_bInMsLevel1 = false;
        }
	else if(isElement("scan", el) && m_bInMsLevel2 == true)
	{
		pushSpectrum();
		m_bInMsLevel2 = false;
	}
	else if(isElement("scan", el) && m_bInMsLevel1 == true && m_bInMsLevel2 == false)
	{
		pushSpectrum();
		m_bInMsLevel1 = false;
	}
}

void SAXMzxmlHandler::characters(const XML_Char *s, int len)
{
	if ((m_bInPeaks && m_msLevel == 2) ||
		(m_bInPrecursorMz))
	{
		m_strData.append(s, len);
	}
	else if (m_bInPeaks && m_msLevel == 1)
	{
		m_strData.append(s, len);
	}
	else if (m_isNameOFile){
		m_filename.append(s,len);
	}
}

void SAXMzxmlHandler::processData()
{
	if( m_bInPeaks && m_msLevel == 2)
	{
		pushPeaks();
	}
	else if( m_bInPeaks && m_msLevel == 1)
	{
		pushPeaks();
	}
	else if (m_bInPrecursorMz)
	{
		m_precursorMz = atof(m_strData.data());
	}

	m_strData.clear();
}
